package im_user

import (
	"context"
	"im-microservice/db"

	"im-microservice/pb"

	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

type ImUserServer struct {
	pb.UnimplementedImUserServer
}

type ImUser struct {
	Name     string
	Mobile   string
	Code     string
	Platform string
}

type ImUserSet struct {
	Accid string
	Mute  bool
}

type ImUserQ struct {
	Accid    string
	Platform string
}

//注册
func (u *ImUserServer) ImUserRegister(ctx context.Context, in *pb.ImUserRequest) (reply *pb.ImUserRegReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	// 是否已经注册过, 如果已经注册返回用户信息
	if c_user, _ := GetImUserByAccidAppkey(in.GetAccid(), in.Common.Appkey); c_user.Accid != "" {
		err = status.Error(10438, "用户已存在")
		return
	}
	// 企业用户注册失败
	c_user, err := CreateImUser(in)
	if err != nil {
		err = status.Error(500, "注册失败")
		return
	}
	reply = &pb.ImUserRegReply{}
	reply.Accid = c_user.Accid
	reply.Name = c_user.Name
	reply.Token = c_user.Token
	return
}

//修改用户信息
func (u *ImUserServer) ImuserSave(ctx context.Context, in *pb.ImUserRequest) (reply *pb.ImUserSetReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	//查询用户是否存在
	c_user, _ := GetImUserByAccidAppkey(in.GetAccid(), in.Common.Appkey)
	if c_user.Token == "" {
		err = status.Error(404, "用户不存在")
		return
	}
	//修改
	m := make(map[string]interface{})
	if in.Name != "" {
		m["name"] = in.Name
	}
	if in.Icon != "" {
		m["icon"] = in.Icon
	}
	if in.Sign != "" {
		m["sign"] = in.Sign
	}
	if in.Email != "" {
		m["email"] = in.Email
	}
	if in.Birth != "" {
		m["birth"] = in.Birth
	}
	if in.Mobile != "" {
		m["mobile"] = in.Mobile
	}
	if in.Gender != 0 {
		m["gender"] = in.Gender
	}
	if in.Ex != "" {
		m["ex"] = in.Ex
	}

	if res, _ := UpdateImUser(in.GetAccid(), in.Common.Appkey, m); res != 1 {
		err = status.Error(500, "修改用户信息失败")
		return
	}
	reply = &pb.ImUserSetReply{}
	return
}

//用户登录
func (u *ImUserServer) ImUserLogin(ctx context.Context, in *pb.ImUserLoginRequest) (reply *pb.ImUserRegReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	//获取用户信息
	c_user, _ := GetImUserByAccidAppkey(in.GetAccid(), in.Common.Appkey)
	if c_user.Token == "" {
		err = status.Error(404, "用户不存在")
		return
	}
	//用户登录处理
	err = LoginHandle(in)
	if err != nil {
		//err = status.Error(500, "用户登录时间更新失败")
		return
	}
	reply = &pb.ImUserRegReply{}
	reply.Token = c_user.Token
	reply.Accid = c_user.Accid
	reply.Name = c_user.Name
	return
}

//用户退出
func (u *ImUserServer) ImUserLoginOut(ctx context.Context, in *pb.ImUserLoginRequest) (reply *pb.ImUserSetReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	c_user, _ := GetImUserByAccidAppkey(in.GetAccid(), in.Common.Appkey)
	if c_user.Accid == "" {
		err = status.Error(404, "用户不存在")
		return
	}
	err = LoginOutHandle(in)
	if err != nil {
		return
	}

	reply = &pb.ImUserSetReply{}
	return reply, nil
}

//用户禁言
func (u *ImUserServer) ImUserForbiddenWords(ctx context.Context, in *pb.ImUserSetRequest) (reply *pb.ImUserSetReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	var mute int64 = 0
	if in.GetMute() == true {
		mute = 1
	}
	//检测该用户
	c_user, err := GetImUserByAccidAppkey(in.GetAccid(), in.Common.Appkey)
	if err != nil {
		err = status.Error(404, "该用户不存在")
		return
	}
	if c_user.MuteEstoppel == mute {
		err = status.Error(417, "请勿重复操作")
		return
	}

	m := make(map[string]interface{})
	m["mute_estoppel"] = mute
	if res, _ := UpdateImUser(in.GetAccid(), in.Common.Appkey, m); res != 1 {
		err = status.Error(500, "用户禁言失败")
		return
	}
	reply = &pb.ImUserSetReply{}
	return
}

//用户禁音视频
func (u *ImUserServer) ImUserForbiddenAV(ctx context.Context, in *pb.ImUserSetRequest) (reply *pb.ImUserSetReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	var mute int64 = 0
	if in.GetMute() == true {
		mute = 1
	}
	//检测该用户
	c_user, err := GetImUserByAccidAppkey(in.GetAccid(), in.Common.Appkey)
	if err != nil {
		err = status.Error(404, "该用户不存在")
		return
	}
	if c_user.MuteAudioVideo == mute {
		err = status.Error(417, "请勿重复操作")
		return
	}

	m := make(map[string]interface{})
	m["mute_audio_video"] = mute
	if res, _ := UpdateImUser(in.GetAccid(), in.Common.Appkey, m); res != 1 {
		err = status.Error(500, "用户禁音视频失败")
		return
	}

	reply = &pb.ImUserSetReply{}
	return
}

//获取用户信息
func (u *ImUserServer) ImUserOne(ctx context.Context, in *pb.ImUserOneRequest) (reply *pb.ImUserOneReply, err error) {
	// 请求超时
	if ctx.Err() == context.Canceled {
		err = status.New(codes.Canceled, "Client cancelled, abandoning.").Err()
		return
	}
	reply = &pb.ImUserOneReply{}
	c_user, err := GetImUserByAccidAppkey(in.GetAccid(), in.Common.Appkey)
	if err != nil {
		return
	}
	reply = getImUser(c_user)
	return
}

func getImUser(imuser db.ImUser) *pb.ImUserOneReply {
	return &pb.ImUserOneReply{
		Id:             int64(imuser.Id),
		Accid:          imuser.Accid,
		Appkey:         imuser.Appkey,
		Name:           imuser.Name,
		Mobile:         imuser.Mobile,
		Gender:         imuser.Gender,
		Valid:          int64(imuser.Valid),
		MuteEstoppel:   int64(imuser.MuteEstoppel),
		MuteAudioVideo: int64(imuser.MuteAudioVideo),
		Ex:             imuser.Ex,
		Createtime:     imuser.Createtime,
		Edition:        imuser.Edition,
	}
}

//获取用户列表
func (u *ImUserServer) ImUserList(ctx context.Context, in *pb.ImUserListRequest) (reply *pb.ImUserListReply, err error) {
	results, err := GetImUserList(in)
	if err != nil {
		return
	}

	reply = &pb.ImUserListReply{Paginate: &pb.Page{}}

	im_user_list := results["list"].([]db.ImUser)
	for _, v := range im_user_list {
		reply.List = append(reply.List, getImUser(v))
	}

	reply.Paginate.PageSize = int64(results["page_size"].(int))
	reply.Paginate.PagetNo = results["page"].(int64)
	reply.Paginate.TotalCount = results["total_count"].(int64)
	reply.Paginate.TotalPage = results["total_page"].(int64)
	return
}
